<?php

namespace DsbNext\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;

class ParametersIndicatorsController {
    
    protected $crudService;
    
    public function __construct($connection, $environment, $service, $util, $crudService) {
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->service      = $service;
        $this->util         = $util;
        $this->crudService	= $crudService;
    }
    
    
   /**
     * Retrieves the unit services.
     *
     * This function retrieves all the unit services based on wich Unit was selected.
     *
     * @param Request\Filter $request The object that receives data from the caller.
     * @param Response $response The object that sends data back to the caller.
     *
     * @return array An array containing the available unit services for that unit.
     *
     * @throws Exception If an error occurs during the retrieval process.
     * 
     * @access public
     * @author Regis Freitas
     */
    public function unitServices(Request\Filter $request, Response $response) {
        $params = ['NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        $data = $this->crudService->getFilteredDataSet('unitServices', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("unitServices", $data));
    } 
    
    
    /**
     * The next 3 functions have the same behavior.
     *
     * This functions retrieves the fields of the tables on DB.
     *
     * @param Request\Filter $request The object that receives data from the caller.
     * @param Response $response The object that sends data back to the caller.
     *
     * @return array An array containing the selected field on the DB.
     *
     * @throws Exception If an error occurs during the retrieval process.
     * 
     * @access public
     * @author Regis Freitas
     */
    public function unitManagerFirstTable(Request\Filter $request, Response $response) {
        $data = $this->service->unitManagerFirstTable();
        $response->addDataSet(new DataSet("unitManagerFirstTable", $data));
    }
    
    public function unitManagerSecondTable(Request\Filter $request, Response $response) {
        $data = $this->service->unitManagerSecondTable();
        $response->addDataSet(new DataSet("unitManagerSecondTable", $data));
    }        
    
    public function aFTrackingTable(Request\Filter $request, Response $response) {
        $data = $this->service->aFTrackingTable();
        $response->addDataSet(new DataSet("aFTrackingTable", $data));
    }    
    
    
    /**
     * The next 2 functions have the same behavior.
     *
     * This functions are creating an array that have the same name of my JSON components, so I can
     * use they on another function later, that will save all these respective fields.
     * 
     * @return array An array containing the names of my JSON components.
     * 
     * @access public
     * @author Regis Freitas
     */
    public function fieldMappingUnitManager() {
        $fieldMapping = [
            'daysOfStashGoal'           => ['daysOfStashGoal', 'daysOfStashMinimumValue'],
            'daysOfDelayLimit'          => ['daysOfDelayLimit', 'daysOfDelayMaximumValue'],
            'valueAdjustmentLimit'      => ['valueAdjustmentLimit', 'valueAdjustmentMaximumValue'],
            'extraRequestsValueLimit'   => ['extraRequestsValueLimit', 'extraRequestsValueMaximumValue'],
            'consumptionDeviationLimit' => ['consumptionDeviationLimit', 'consumptionDeviationMaximumValue'],
            'daysToApproveMenuLimit'    => ['daysToApproveMenuLimit', 'daysToApproveMenuMaximumValue']
        ];
        
        return $fieldMapping;
    }
    
    public function fieldMappingAfTracking() {
        $fieldMapping = [
            'AfCriticalValue'      => ['AfCriticalValue', 'AfGoal'],
            'AfAnnualDeliveryGoal' => ['AfAnnualDeliveryGoal', 'AfEmptyField']
        ];
        
        return $fieldMapping;
    }
    
    
    /**
     * A function to save the status of a indicator on the 'servicesGrid'
     *
     * This functions retrieves all the edited rows on the grid, and then will check if the fields
     * "CDSERVEFILI" and "CDFILIAL" are matching, and then replace the value of the "IDINDICADOR".
     *
     * @param Request\Filter $request The object that receives data from the caller.
     * @param Response $response The object that sends data back to the caller.
     *
     * @return array An array containing fields of the DB.
     *
     * @throws Exception If an error occurs during the retrieval process.
     * 
     * @access public
     * @author Regis Freitas
     */
    public function saveOnDashboard(Request\DataSet $request, Response $response) {
        $rows = $request->getDataSet()->getRows();
        try{
            $this->service->beginTransaction();
            
            foreach($rows as $row){
                $params = [
                    "P_CDSERVFILI"  => $row["CDSERVFILI"],
                    "P_CDFILIAL"    => $row["CDFILIAL"],
                    "P_IDINDICADOR" => $row["IDINDICADOR"]
                ];
                $this->service->updateShowOnDashboardStatus($params);
            }
            $this->service->commit();
        }catch(Exception $e){ 
            $this->service->rollback();
            throw $e;
        }
    }
    
    
    /**
     * This functions will edit two different tables on DB.
     *
     * The first array($paramGestaoTable) receive the values retrieved from the front-end, and then
     * change the actual value.
     * 
     * The second array($indicatorTfTable) is inside a foreach, which will repeat as much as the 
     * size of $fieldMapping, populating the paramaters VRLIMMETA and VRLIMMINIMO with the values
     * retrived from the front-end.
     * 
     * The function saveAfTracking works the same way as saveUnitManager second array.
     *
     * @param Request\Filter $request The object that receives data from the caller.
     * @param Response $response The object that sends data back to the caller.
     *
     * @return array An array containing fields of the DB.
     *
     * @throws Exception If an error occurs during the retrieval process.
     * 
     * @access public
     * @author Regis Freitas
     */
    public function saveUnitManager(Request\Row $request, Response $response) {
        $rows = $request->getParameters();
        try{
            $this->service->beginTransaction();
            
            $paramGestaoTable = [
                "P_CDGRUPOPROD"     => $rows["productGrouping"],
                "P_IDCONSXMLCOMFIL" => $rows["xmlSwitch"],
                "P_NRORG"           => $this->environment->getNrorg()
            ];
            $this->service->saveUnitManagerFirstTable($paramGestaoTable);
            
            $fieldMapping = $this->fieldMappingUnitManager();

            foreach ($fieldMapping as $mainField => $subFields) {
                $mainRow = $rows[$mainField];
                $subRow = $rows[$subFields[1]];
                $indicadorTfTable = [
                    "P_VRLIMMETA"       => $mainRow[1],
                    "P_VRLIMMINIMO"     => $subRow[1],
                    "P_NRINDICADORTF"   => $mainRow[0],
                    "P_NRORG"           => $this->environment->getNrorg()
                    
                ];
                $this->service->saveUnitManagerSecondTable($indicadorTfTable);
            }

            $this->service->commit();
        }catch(Exception $e){ 
            $this->service->rollback();
            throw $e;
        }
    }
    
    public function saveAfTracking(Request\Row $request, Response $response) {
        $rows = $request->getParameters();
        
        try{
            $this->service->beginTransaction();
            
            $fieldMapping = $this->fieldMappingAfTracking();

            foreach ($fieldMapping as $mainField => $subFields) {
                $mainRow = $rows[$mainField];
                $subRow = $rows[$subFields[1]];
                $indicadorTfTable = [
                    "P_VRLIMMETA"       => $mainRow[1],
                    "P_VRLIMMINIMO"     => $subRow[1],
                    "P_NRINDICADORTF"   => $mainRow[0],
                    "P_NRORG"           => $this->environment->getNrorg()
                    
                ];
                $this->service->saveAfTracking($indicadorTfTable);
            }
            $this->service->commit(); 
        }catch(Exception $e){ 
            $this->service->rollback();
            throw $e;
        }
    }
    
    public function checkingTableParameters(Request\Filter $request, Response $response) {
        $data = $this->service->checkingTableParameters();
        if (empty($data) && !isset($data[0]['CDPARAMGESTAO'],$data[0]['TAXAADMINMATRIZ'])) {
            var_dump('teste');
            try{
                $params = [
                    "CDPARAMGESTAO"   => '0001',
                    "TAXAADMINMATRIZ" => 0,
                    "NRORG"           => $this->environment->getNrorg()
                ];
                $this->service->beginTransaction();
                $this->service->insertTableParameters($params);
                $this->service->commit(); 
            }catch(Exception $e){ 
                $this->service->rollback();
                throw $e;
            }    
        }
    } 
    
}